
/*
Copyright 2007-2009 Kim A. Brandt <kimabrandt@gmx.de>

This file is part of yarip.

Yarip is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Yarip is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with yarip; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

function YaripList(name, exclusive)
{
    this.name = "";
    this.obj = {};
    this.length = 0;
    this.exclusive = false;

    this.setName = function(v)
    {
        if (v) this.name = "" + v;
    }

    this.setExclusive = function(v)
    {
        this.exclusive = "" + v == "true";
    }

    this.getExclusive = function()
    {
        return this.length > 0 && this.exclusive;
    }

    this.add = function(o, purge)
    {
        if (!o) return;
        var oldObj = this.obj[o.getKey()];
        if (oldObj)
        {
            oldObj.merge(o);
            if (purge) oldObj.purge();
        }
        else
        {
            this.length++;
            if (purge) o.purge();
            this.obj[o.getKey()] = o;
        }
    }

    this.remove = function(v)
    {
        if (!v || !this.obj[v]) return;
        delete this.obj[v];
        this.length--;
    }

    this.reset = function()
    {
        this.obj = {};
        this.length = 0;
    }

    this.sort = function(purge)
    {
        var a = [];
        for (var k in this.obj) a.push(this.obj[k]);
        a.sort(this.sortBy);
        this.reset();
        for (var i = 0; i < a.length; i++) this.add(a[i], purge);
    }

    this.clone = function(purge)
    {
        var tmp = new this.constructor(this.name, this.exclusive);
        for (var k in this) if (this[k] && typeof(this[k]) != "object" && !/^(name|length|exclusive)$/.test(k)) tmp[k] = this[k];
        for (var v in this.obj) tmp.add(this.obj[v].clone(purge));
        return tmp;
    }

    this.merge = function(o)
    {
        this.setExclusive(this.getExclusive() || o.getExclusive());
        for (var v in o.obj) this.add(o.obj[v].clone());
    }

    this.generateXml = function()
    {
        var tmp = "";
        for (var v in this.obj) tmp += this.obj[v].generateXml();
        if (tmp != "") return "\t\t" +
            "<" + this.name +
                (this.exclusive ? " exclusive=\"" + this.exclusive + "\"" : "") +
            ">\n" + tmp + "\t\t</" + this.name + ">\n";
        else return "";
    }

    this.sortBy = function(a, b)
    {
        if (a.getKey() < b.getKey()) return -1;
        if (a.getKey() > b.getKey()) return 1;
        return 0;
    }

    this.setName(name);
    this.setExclusive(exclusive);
}

function YaripElementList()
{
    this.name = "";
    this.obj = {};
    this.length = 0;
}
YaripElementList.prototype = new YaripList;
YaripElementList.prototype.reduce = function()
{
    var a = [];
    for (var k in this.obj) a.push(this.obj[k]);
    this.reset();

    var r = [];
    var t = [];
    var o = a.shift();
    while (o)
    {
        r.push(o);
//        var noCheck = !/^\/?(\/([a-z][\w]*|\*)(\[@[a-z][\w]*='[^']*'(\ and\ )?\])*)+$/i.test(o.getXPath());
        var noCheck = !/^\/?(\/[a-z]\w*(\[(\d+|@[a-z]\w*='[^']*?'(\sand\s)?)+\])*)+$/i.test(o.getXPath());
        for (var i = 0; i < a.length; i++)
        {
            if (noCheck                                                // do not check
                || !/^\/?(\/[a-z]\w*(\[(\d+|@[a-z]\w*='[^']*?'(\sand\s)?)+\])*)+$/i.test(a[i].getXPath()) // wasn't auto generated
                || a[i].getXPath().indexOf(o.getXPath()) != 0          // does not contain
                || a[i].getXPath().charAt(o.getXPath().length) != "/"  // is not the end of a sub-path
               ) t.push(a[i]);
        }
        a = t;
        t = [];
        o = a.shift();
    }
    a = r;

    for (var i = 0; i < a.length; i++) this.add(a[i]);
}

function YaripElementWhitelist()
{
    this.name = "elementWhitelist";
    this.obj = {};
    this.length = 0;
    this.exclusive = false;
}
YaripElementWhitelist.prototype = new YaripElementList;
YaripElementWhitelist.prototype.toArray = function()
{
    var a = [];
    for (var v in this.obj)
    {
        var o = this.obj[v];
        a[a.length] = [v, o.getDeep(), o.getFound(), o.getNotFound(), o.getIgnored()];
    }
    return a;
}

function YaripElementBlacklist()
{
    this.name = "elementBlacklist";
    this.obj = {};
    this.length = 0;
}
YaripElementBlacklist.prototype = new YaripElementList;
YaripElementBlacklist.prototype.toArray = function()
{
    var a = [];
    for (var v in this.obj)
    {
        var o = this.obj[v];
        a[a.length] = [v, o.getDeep(), o.getForce(), o.getDOW(), o.getFound(), o.getNotFound(), o.getIgnored()];
    }
    return a;
}
YaripElementBlacklist.prototype.generateCSS = function(notDeep)
{
    var tmp = "";
    for (var v in this.obj) tmp += this.obj[v].generateCSS(notDeep);
    if (tmp != "") return tmp.replace(/,(\n)$/, "$1") + "\t{\n\t\tdisplay: none !important;\n\t}\n";
    else return "";
}

function YaripContentWhitelist()
{
    this.name = "contentWhitelist";
    this.obj = {};
    this.length = 0;
    this.exclusive = false;
}
YaripContentWhitelist.prototype = new YaripList;
YaripContentWhitelist.prototype.toArray = function()
{
    var a = [];
    for (var k in this.obj)
    {
        var o = this.obj[k];
        a[a.length] = [/*o.getType(),*/ o.getRegExp(), o.getDeep(), o.getFound()];
    }
    return a;
}

function YaripContentBlacklist()
{
    this.name = "contentBlacklist";
    this.obj = {};
    this.length = 0;
}
YaripContentBlacklist.prototype = new YaripList;
YaripContentBlacklist.prototype.toArray = function()
{
    var a = [];
    for (var k in this.obj)
    {
        var o = this.obj[k];
        a[a.length] = [/*o.getType(),*/ o.getRegExp(), o.getDeep(), o.getForce(), o.getFound(), o.getIgnored()];
    }
    return a;
}

function YaripStylesList()
{
    this.name = "styles";
    this.obj = {};
    this.length = 0;
}
YaripStylesList.prototype = new YaripList;
YaripStylesList.prototype.toArray = function()
{
    var a = [];
    for (var v in this.obj)
    {
        var o = this.obj[v];
        a[a.length] = [v, o.style, o.getDeep(), o.getFound(), o.getNotFound()];
    }
    return a;
}
YaripStylesList.prototype.generateCSS = function(notDeep)
{
    var tmp = "";
    for (var v in this.obj)
    {
//        if (!this.obj[v].getForce()) continue;
        if (!notDeep != this.obj[v].getDeep() || !this.obj[v].getForce()) continue;
        var xpathAsStyle = this.obj[v].xpathToStyle();
        if (xpathAsStyle) tmp += "\t" + xpathAsStyle + "\n\t{\n\t\t" + this.obj[v].style.replace(/\s*;(\s*\*\/)?\s*/g, ";$1\n\t\t").replace(/\n\t\t$/, "") + "\n\t}\n";
    }
    return tmp;
}

function YaripExtList()
{
    this.name = "extension";
    this.obj = {};
    this.length = 0;
}
YaripExtList.prototype = new YaripList;
YaripExtList.prototype.toArray = function()
{
    var a = [];
    for (var k in this.obj) a[a.length] = [k];
    return a;
}
