
/*
Copyright 2007-2009 Kim A. Brandt <kimabrandt@gmx.de>

This file is part of yarip.

Yarip is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Yarip is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with yarip; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

function YaripMonitorDialog()
{
    this.gBrowser = null;
    this.stringbundle = null;
    this.tree = null;
    this.doScroll = true;

    this.openAddress = function()
    {
        if (this.tree.currentIndex < 0) return;

//        var item = this.tree.view.getItem(this.tree.currentIndex);
        var item = yaripMonitorTreeView.getItem(this.tree.currentIndex);
        var contentLocation = item.contentLocation;
        if (!contentLocation) return;

        this.gBrowser.selectedTab = this.gBrowser.addTab(contentLocation);
    }

    this.copyAddress = function()
    {
        if (this.tree.currentIndex < 0) return;

//        var item = this.tree.view.getItem(this.tree.currentIndex);
        var item = yaripMonitorTreeView.getItem(this.tree.currentIndex);
        var contentLocation = item.contentLocation;
        if (!contentLocation) return;

        var clipboardHelper = Components.classes["@mozilla.org/widget/clipboardhelper;1"].getService(Components.interfaces.nsIClipboardHelper);
        clipboardHelper.copyString(contentLocation);
    }

    this.whitelistContent = function()
    {
        if (this.tree.currentIndex < 0) return;

//        var item = this.tree.view.getItem(this.tree.currentIndex);
        var item = yaripMonitorTreeView.getItem(this.tree.currentIndex);
        var address = item.address;
        if (!address) return;

        var regExp = yarip.generateRegExp(item.contentLocation);
        if (!regExp) return;

        var obj = {
            content: new YaripContent(null, regExp)
        }

        window.openDialog("chrome://yarip/content/whitelistcontentdialog.xul", "whitelistcontentdialog", "chrome,modal,resizable", obj);
        if (!obj.content) return;

        if (yarip.whitelistContent(address, obj.content.getRegExp(), obj.content))
        {
            yarip.map.sort(false);
            yarip.setChangeType(yarip.ESSENTIAL_CHANGE);
//            yarip.setValue(yarip.RELOAD, new Date().getTime(), yarip.STRING); // notify
            yarip.pageDialog.reloadPage(address);
        }
    }

    this.blacklistContent = function()
    {
        if (this.tree.currentIndex < 0) return;

//        var item = this.tree.view.getItem(this.tree.currentIndex);
        var item = yaripMonitorTreeView.getItem(this.tree.currentIndex);
        var address = item.address;
        if (!address) return;

        var regExp = yarip.generateRegExp(item.contentLocation);
        if (!regExp) return;

        var obj = {
            content: new YaripContent(null, regExp)
        }

        window.openDialog("chrome://yarip/content/blacklistcontentdialog.xul", "blacklistcontentdialog", "chrome,modal,resizable", obj);
        if (!obj.content) return;

        if (yarip.blacklistContent(address, obj.content.getRegExp(), obj.content))
        {
            yarip.map.sort(false);
            yarip.setChangeType(yarip.ESSENTIAL_CHANGE);
//            yarip.setValue(yarip.RELOAD, new Date().getTime(), yarip.STRING); // notify
            yarip.pageDialog.reloadPage(address);
        }
    }

    this.logContent = function(status, address, contentLocation)
    {
        var item = {}
        item.address = address;
        item.time = new Date().toLocaleString();
        item.status = status;
        item.contentLocation = contentLocation;

//        this.tree.view.appendItem(item, this.doScroll);
        yaripMonitorTreeView.appendItem(item, this.doScroll);
    }

    this.clear = function()
    {
//        this.tree.view.clearItems();
        yaripMonitorTreeView.clearItems();
    }

    this.handleEvent = function(event)
    {
        if (!event) return;

        switch(event.target.id)
        {
            case "content-tree":
//                this.doScroll = this.tree.view.rowCount == this.tree.currentIndex + 1;
//                this.doScroll = this.tree.view.rowCount == this.tree.currentIndex + 1;
                this.doScroll = yaripMonitorTreeView.rowCount == this.tree.currentIndex + 1;
        }
    }

    this.load = function()
    {
        if (!"arguments" in window || window.arguments.length < 1) return;

        this.gBrowser = window.arguments[0];
        this.stringbundle = document.getElementById("monitor-dialog-stringbundle");
        this.tree = document.getElementById("content-tree");
        this.tree.view = yaripMonitorTreeView;
//        this.tree.view = new YaripMonitorTreeView();
//        this.tree.view = new YaripTreeView(this.tabs[tab].listArray, this.tabs[tab].columns, this.tabs[tab].listArray.length);
        this.tree.addEventListener("select", this, false);
        yarip.monitorDialog = this;
    }

    this.unload = function()
    {
        yarip.monitorDialog = null;
    }
}

var yaripMonitorTreeView = {
    treebox: null,
    childData: [],
    visibleData: [],
    whitelisted: Components.classes["@mozilla.org/atom-service;1"].getService(Components.interfaces.nsIAtomService).getAtom("whitelisted"),
    blacklisted: Components.classes["@mozilla.org/atom-service;1"].getService(Components.interfaces.nsIAtomService).getAtom("blacklisted"),

    get rowCount() { return this.visibleData.length; },

    cycleHeader: function(col) {},
    getCellProperties: function(row, col, properties)
    {
        if (col.id != "status") return;
        if (row < 0 || row >= this.visibleData.length) return;

        var item = this.visibleData[row];
        var status = item.status;
        if (status > 0)
        {
            switch (status)
            {
                case 1: properties.AppendElement(this.whitelisted); break;
                case 2: properties.AppendElement(this.blacklisted); break;
            }
        }
    },
    getCellText: function(row, col)
    {
        var colid = col.id;
        if (colid == "status") return "";
        if (row < 0 || row >= this.visibleData.length) return "";

        var item = this.visibleData[row];
        return (colid in item ? item[colid] : null);
    },
    canDrop: function(index, orientation) { return false; },
    cycleCell: function(row, col) {},
    drop: function(row, orientation) { return 0; },
    getCellValue: function(row, col) { return ""; },
    getColumnProperties: function(col, properties) {},
    getImageSrc: function(row, col) { return ""; },
    getLevel: function(index) { return 0; },
    getParentIndex: function(rowIndex) { return -1; },
    getProgressMode: function(row, col) { return false; },
    getRowProperties: function(index, properties) {},
    hasNextSibling: function(rowIndex, afterIndex) { return false; },
    isContainerEmpty: function(index) { return false; },
    isContainer: function(index) { return false; },
    isContainerOpen: function(index) { return false; },
    isEditable: function(row, col) { return false; },
    isSelectable: function(row, col) { return false; },
    isSeparator: function(index) { return false; },
    isSorted: function() { return false; },
    performAction: function(action) {},
    performActionOnCell: function(action, row, col) {},
    performActionOnRow: function(action, row) {},
    selectionChanged: function() {},
    setCellText: function(row, col, value) {},
    setCellValue: function(row, col, value) {},
    setTree: function(treebox) { this.treebox = treebox; },
    toggleOpenState: function(index) {},

    appendItem: function(item, doScroll)
    {
        this.childData.push(item);
        var index = this.visibleData.length;
        this.visibleData.splice(index, 0, item);
        this.treebox.rowCountChanged(index, 1);
        if (doScroll) this.treebox.ensureRowIsVisible(index);
    },

    getItem: function(currentIndex)
    {
        return this.visibleData[currentIndex];
    },

    clearItems: function()
    {
        var oldRows = this.rowCount;

        this.childData = [];
        this.visibleData = [];
        this.treebox.rowCountChanged(0, -oldRows);
    }
}

//function YaripMonitorTreeView()
//{
//    this.treebox = null;
//    this.childData = [];
//    this.visibleData = [];
//    this.whitelisted = Components.classes["@mozilla.org/atom-service;1"].getService(Components.interfaces.nsIAtomService).getAtom("whitelisted");
//    this.blacklisted = Components.classes["@mozilla.org/atom-service;1"].getService(Components.interfaces.nsIAtomService).getAtom("blacklisted");

//    this.rowCount() { return this.visibleData.length; }

//    this.cycleHeader = function(col) {}
//    this.getCellProperties = function(row, col, properties)
//    {
//        if (col.id != "status") return;
//        if (row < 0 || row >= this.visibleData.length) return;

//        var item = this.visibleData[row];
//        var status = item.status;
//        if (status > 0)
//        {
//            switch (status)
//            {
//                case 1: properties.AppendElement(this.whitelisted); break;
//                case 2: properties.AppendElement(this.blacklisted); break;
//            }
//        }
//    }
//    this.getCellText = function(row, col)
//    {
//        var colid = col.id;
//        if (colid == "status") return "";
//        if (row < 0 || row >= this.visibleData.length) return "";

//        var item = this.visibleData[row];
//        return (colid in item ? item[colid] : null);
//    }
//    this.getColumnProperties = function(col, properties) {}
//    this.getImageSrc = function(row, col) { return ""; }
//    this.getLevel = function(index) { return 0; }
//    this.getRowProperties = function(index, properties) {}
//    this.isContainer = function(index) { return false; }
//    this.isSeparator = function(index) { return false; }
//    this.isSorted = function() { return false; }
//    this.setTree = function(treebox) { this.treebox = treebox; }

//    this.appendItem = function(item, doScroll)
//    {
//        this.childData.push(item);
//        var index = this.visibleData.length;
//        this.visibleData.splice(index, 0, item);
//        this.treebox.rowCountChanged(index, 1);
//        if (doScroll) this.treebox.ensureRowIsVisible(index);
//    }

//    this.getItem = function(currentIndex)
//    {
//        return this.visibleData[currentIndex];
//    }

//    this.clearItems = function()
//    {
//        var oldRows = this.rowCount;

//        this.childData = [];
//        this.visibleData = [];
//        this.treebox.rowCountChanged(0, -oldRows);
//    }
//}
